
//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------

Texture2D overlay;        
float roi_x;
float roi_y;
float    g_fTime;                   
int overlay_Width;
int overlay_Height;
int dst_Width;
int dst_Height;


//--------------------------------------------------------------------------------------
// Texture samplers
//--------------------------------------------------------------------------------------
SamplerState TextureSampler
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

struct VS_INPUT
{
    float4 Position   : POSITION; // vertex position 
    float2 TextureUV  : TEXCOORD0;   // vertex texture coords 
};

struct VS_OUTPUT
{
    float4 Position   : SV_POSITION; // vertex position 
    float2 TextureUV  : TEXCOORD1;   // vertex texture coords 
};

//--------------------------------------------------------------------------------------
// Pixel shader output structure
//--------------------------------------------------------------------------------------
struct PS_OUTPUT
{
    float4 RGBColor : SV_Target;  // Pixel color
};


//--------------------------------------------------------------------------------------
// This shader outputs the pixel's color by modulating the texture's
//       color with diffuse material color
//--------------------------------------------------------------------------------------
float2 rotateFunc(float2 uv, float2 center, float theta)
{
	float2 temp;
	temp.x = dot(float2(cos(theta), -sin(theta)), uv - center);
	temp.y = dot(float2(sin(theta), cos(theta)), uv - center);
	return (temp + center);
}

PS_OUTPUT PS_2D( VS_OUTPUT In)
{ 
    PS_OUTPUT Output;
    float ow = float(overlay_Width);
    float oh = float(overlay_Height);
	float dstW = float(dst_Width);
	float dstH = float(dst_Height);
	  
    float2 tc = In.TextureUV;
	float u = tc.x * dstW / ow + roi_x;
	float v = tc.y * dstH / oh + roi_y;
    
    float2 uv = float2(u,v);
    
    float grid = (step(0.0, uv.x) - step(1.0,uv.x)) * (step(0.0, uv.y) - step(1.0, uv.y));   
	float4 ovlCol = overlay.Sample(TextureSampler, uv) * grid;
	
	Output.RGBColor = ovlCol;
    return Output;
}

// 
VS_OUTPUT VS_2D(VS_INPUT vin)
{
	VS_OUTPUT vOut;
	vOut.Position = vin.Position;
	vOut.TextureUV = vin.TextureUV;
	return vOut;
}

//--------------------------------------------------------------------------------------
// Renders scene to render target using D3D11 Techniques
//--------------------------------------------------------------------------------------
technique11 ColorTech
{
	pass P0
	{
		SetVertexShader(CompileShader(vs_4_0, VS_2D()));
		SetPixelShader(CompileShader(ps_4_0, PS_2D()));

	}
}

