


//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
Texture2D overlay;            // Color texture for mesh
float    g_fTime;                   // App's time in seconds
int overlay_Width;
int overlay_Height;
int back_Width;
int back_Height;


//--------------------------------------------------------------------------------------
// Texture samplers
//--------------------------------------------------------------------------------------
SamplerState TextureSampler
{
    Filter = MIN_MAG_MIP_NEAREST;
    AddressU = Wrap;
    AddressV = Wrap;
};

struct VS_INPUT
{
    float4 Position   : POSITION; // vertex position 
    float2 TextureUV  : TEXCOORD0;   // vertex texture coords 
};

struct VS_OUTPUT
{
    float4 Position   : SV_POSITION; // vertex position 
    float2 TextureUV  : TEXCOORD1;   // vertex texture coords 
};

//--------------------------------------------------------------------------------------
// Pixel shader output structure
//--------------------------------------------------------------------------------------
struct PS_OUTPUT
{
    float4 RGBColor : SV_Target;  // Pixel color
};


//--------------------------------------------------------------------------------------
// This shader outputs the pixel's color by modulating the texture's
//       color with diffuse material color
//--------------------------------------------------------------------------------------
float3 NV12ToRGB_CSC(float3 rgb)
{
    float r = rgb.x;
    float g = rgb.y;
    float b = rgb.z;	    
    float y = 0.182586 * r + 0.628254 * g + 0.063423 * b + 16.0;
    float u = -0.098397 * r - 0.338572 * g + 0.439216 * b + 128.0;
    float v = 0.429412 * r - 0.398942 * g - 0.040274 * b + 128.0;
    return (float3)(y,u,v);
}
#define EQN_EPS 1e-9f

static bool isZero(float x) {
	return (x > -EQN_EPS && x < EQN_EPS);
}

PS_OUTPUT PS_2D( VS_OUTPUT In)
{ 
    PS_OUTPUT Output;
    float ow = float(overlay_Width);
    float oh = float(overlay_Height);
    float bw = float(back_Width);
    float bh = float(back_Height);
    float2 tc = In.TextureUV;
    int w = int(tc.x * overlay_Width);
    int h = int(tc.y * overlay_Height);
    // float one_pixelX = 1.0 / ow;
    // float one_pixelY = 1.0 / oh;
    
    float4 InRGBAColor = overlay.Sample(TextureSampler, tc) * 255.0;
    float3 yuv = NV12ToRGB_CSC(InRGBAColor.xyz) / 255.0;	
 
	
    Output.RGBColor = float4(yuv,0.0);

    return Output;

}

// 
VS_OUTPUT VS_2D(VS_INPUT vin)
{
	VS_OUTPUT vOut;
	vOut.Position = vin.Position;
	vOut.TextureUV = vin.TextureUV;
	return vOut;
}

//--------------------------------------------------------------------------------------
// Renders scene to render target using D3D11 Techniques
//--------------------------------------------------------------------------------------
technique11 ColorTech
{
	pass P0
	{
		SetVertexShader(CompileShader(vs_4_0, VS_2D()));
		SetPixelShader(CompileShader(ps_4_0, PS_2D()));

	}
}

